import sys
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String, Float, Unicode
from sqlalchemy.orm import sessionmaker
from sqlalchemy import create_engine
from optparse import OptionParser
from optparse import SUPPRESS_USAGE
import pygeoclients

Base = declarative_base()

def reset_database(engine):
   Base.metadata.drop_all(engine)
   Base.metadata.create_all(engine)

def create_factories(connection_string):
   engine = create_engine(connection_string)
   return (engine, sessionmaker(bind=engine))

class AddressGeolocation(Base):
   __tablename__ = "address_geolocation"

   id        = Column(Integer, primary_key=True)
   address   = Column(String(200), nullable=False)
   lat       = Column(Float, nullable=True)
   lng       = Column(Float, nullable=True)
   accuracy  = Column(Integer, nullable=True)
   info_data = Column(String(500), nullable=True)

   def __init__(self, address, info_data=None):
      self.address   = address
      self.info_data = info_data

   def __repr__(self):
      return "<AddressGeolocation('%s', '%s', '%s', '%s', '%s', '%s')>" % (self.id,
             self.address, self.lat, self.lng, self.accuracy, self.info_data)


if __name__ == "__main__":
   parser = OptionParser(usage=SUPPRESS_USAGE)
   print "Geocoding Database v.%s\nBy %s\n" % (pygeoclients.__version__, pygeoclients.__author__)
   parser.add_option("-f", "--file", dest="dbfile",
                  help="Database to read (default is 'sqlite:///db.db').",
                  metavar="FILENAME", default="sqlite:///db.db")
   parser.add_option("-s", "--show", dest="show", action="store_true",
                  help="Show all records in database.", default=False)

   parser.add_option("-a", "--add", dest="add", metavar="ADDRESS",
                  help="Add a new address record.")

   parser.add_option("-d", "--delete", dest="delete", metavar="ADDRESS",
                  help="Delete a address record, it will find for \%ADDRESS\%.")

   parser.add_option("-r", "--reset", dest="reset", action="store_true",
                  help="Reset all records in the database, drop and recreate all tables.", default=False)
   

   (options, args) = parser.parse_args()

   ENGINE, SESSION_FACTORY = create_factories(options.dbfile)
   session = SESSION_FACTORY()

   if not (options.show or options.add or options.delete or options.reset):
      parser.print_help()
      sys.exit()

   if options.reset:
      print "Reseting database..."
      reset_database(ENGINE)
      session.close()
      print "Done !"
      sys.exit()

   if options.show:
      print "Showing records..."
      record_count = 0
      query = session.query(AddressGeolocation)
      for row in query:
         print "%s" % row
         record_count += 1
      session.close()
      print "%d records on database." % record_count
      sys.exit()

   if options.add:
      new_rec = AddressGeolocation(options.add)
      session.add(new_rec)
      session.commit()
      session.close()
      print "Address [%s] ok !" % options.add
      sys.exit()

   if options.delete:
      query = session.query(AddressGeolocation).filter(AddressGeolocation.address.like("%%%s%%" % options.delete))
      delete_count = 0
      for rec in query:
         print "Removing [%s] address..." % rec.address,
         session.delete(rec)
         delete_count += 1
         print "done !"
      print "%d records deleted." % delete_count
      session.commit()
      session.close()
      sys.exit()


