import time
import sys
import urllib, urllib2
from optparse import OptionParser
import pygeoclients
from pygeoclients import db
from optparse import SUPPRESS_USAGE

GEOCODE_URL = "http://maps.google.com/maps/geo?"
GEOCODE_CODES = {
   "200" : "SUCCESS",
   "400" : "BAD REQUEST",
   "500" : "SERVER ERROR",
   "601" : "MISSING QUERY",
   "602" : "UNKOWN ADDRESS",
   "603" : "UNAVAILABLE ADDRESS",
   "604" : "UNKOWN DIRECTIONS",
   "610" : "BAD KEY",
   "620" : "TOO MANY QUERIES"
}

GEOCODE_ACCURACY = {
   "0" : "Unknown accuracy",
   "1" :	"Country level accuracy",
   "2" :	"Region (state, province, prefecture, etc.) level accuracy",
   "3" :	"Sub-region (county, municipality, etc.) level accuracy",
   "4" :	"Town (city, village) level accuracy",
   "5" :	"Post code (zip code) level accuracy",
   "6" :	"Street level accuracy",
   "7" :	"Intersection level accuracy",
   "8" :	"Address level accuracy",
   "9" :	"Premise (building name, property name, shopping center, etc.) level accuracy"
}

def geocode(address, google_key="", output_format="csv"):
   parameters = {"q"      : address.encode("utf-8"),
                 "output" : output_format,
                 "key"    : google_key}

   data = urllib.urlencode(parameters)
   url  = GEOCODE_URL + data
   req  = urllib2.Request(url)
   response = urllib2.urlopen(req)
   geodat   = response.read().split(",")
   response.close()
   code = geodat[0]
   if code == "200":
      code, accuracy, lat, lng = geodat
      return {"code"     : code,
              "accuracy" : accuracy,
              "lat"      : lat,
              "lng"      : lng}
   else: return {"code":code}

if __name__ == "__main__":
   parser = OptionParser(usage=SUPPRESS_USAGE)
   print "Geocoding Service v.%s\nBy %s\n" % (pygeoclients.__version__, pygeoclients.__author__)
   parser.add_option("-f", "--file", dest="dbfile",
                  help="Database to read (default is 'sqlite:///db.db').",
                  metavar="FILENAME", default="sqlite:///db.db")
   parser.add_option("-i", "--interval", dest="interval",
                  help="Interval (in seconds) between calls of Google service (default is '2.0' seconds).",
                  metavar="INTERVAL", default="2.0")
   parser.add_option("-r", "--reprocess", dest="reprocess", action="store_true",
                  help="Force the processing of all records.", default=False)

   (options, args) = parser.parse_args()
   print "Loading database..."
   engine, session_factory = db.create_factories(options.dbfile)
   session = session_factory()

   for row in session.query(db.AddressGeolocation).filter(db.AddressGeolocation.accuracy == None):
      print "Geocoding the address [%s]..." % row.address
      time.sleep(float(options.interval))
      geo_data = geocode(row.address)
      if geo_data["code"] != "200":
         print "\tError: [%s]." % GEOCODE_CODES[geo_data["code"]]
         row.accuracy = -1
         session.add(row)
      else:
         print "\tAccuracy: %s." % GEOCODE_ACCURACY[geo_data["accuracy"]]
         row.lat = geo_data["lat"]
         row.lng = geo_data["lng"]
         row.accuracy = geo_data["accuracy"]
         session.add(row)

   session.commit()
   session.close()

   print "Done !"
