import time
from optparse import OptionParser
import BaseHTTPServer
import pygeoclients
from pygeoclients import db
from pygeoclients import util
from optparse import SUPPRESS_USAGE

try:
   import json
except ImportError:
   import simplejson as json

ENGINE, SESSION_FACTORY = None, None

class GeoMapHandler(BaseHTTPServer.BaseHTTPRequestHandler):
   def do_GET(self):
      global SESSION_FACTORY
      session = SESSION_FACTORY()
      self.send_response(200)

      parse = util.urlparse_qs(self.path)
      s, w = parse["sw"][0].split(",")
      n, e = parse["ne"][0].split(",")

      query = session.query(db.AddressGeolocation).filter(
         db.AddressGeolocation.lat <= n).filter(
         db.AddressGeolocation.lat >= s).filter(
         db.AddressGeolocation.lng <= e).filter(
         db.AddressGeolocation.lng >= w)

      json_prepare = []
      for q in query:
         d = q.__dict__.copy()
         del d["_sa_instance_state"]
         json_prepare.append(d)
              
      self.send_header("Content-type", "text/html")
      self.end_headers()
      self.wfile.write(json.dumps(json_prepare))

      session.close()

   def log_request(self, code):
      print '.' if code==200 else '#',

class GeoEarthHandler(BaseHTTPServer.BaseHTTPRequestHandler):
   def do_GET(self):
      global SESSION_FACTORY
      session = SESSION_FACTORY()
      self.send_response(200)
      
      query = session.query(db.AddressGeolocation).filter(db.AddressGeolocation.lat != None)

      kml_begin = (
         '<?xml version="1.0" encoding="UTF-8"?>\n'
         '<kml xmlns="http://www.opengis.net/kml/2.2">\n'
         '<Document>\n'
      )

      placemark = (
         '<Placemark>\n'
         '<name>%s</name>\n'
         '<Point>\n'
         '<coordinates>%s,%s</coordinates>\n'
         '</Point>\n'
         '</Placemark>\n'
      )

      kml_end = (
         '</Document>'
         '</kml>'
      )

      self.send_header("Content-type", "application/vnd.google-earth.kml+xml")
      self.end_headers()

      self.wfile.write(kml_begin)
      for row in query:
         self.wfile.write(placemark % (row.info_data, row.lng, row.lat))
      self.wfile.write(kml_end)

      session.close()

   def log_request(self, code):
      print '.' if code==200 else '#',

if __name__ == '__main__':
   parser = OptionParser(usage=SUPPRESS_USAGE)
   print "Geoserver v.%s\nBy %s\n" % (pygeoclients.__version__, pygeoclients.__author__)
   parser.add_option("-f", "--file", dest="dbfile",
                     help="Database to read (default is 'sqlite:///db.db').",
                     metavar="FILENAME", default="sqlite:///db.db")
   parser.add_option("--host", "--host", dest="host",
                     help="Host to start server (default is 'localhost').",
                     metavar="HOST", default="localhost")
   parser.add_option("-p", "--port", dest="port",
                     help="Port to listen (default is '6666').",
                     metavar="PORT", default="6666")
   parser.add_option("-m", "--map", dest="map",
                     help="Use the Google Maps server mode (the default).",
                     default=True, action="store_true")
   parser.add_option("-e", "--earth", dest="earth",
                     help="Use the Google Earth server mode.",
                     default=False, action="store_true")

   (options, args) = parser.parse_args()

   print "Loading database..."
   ENGINE, SESSION_FACTORY = db.create_factories(options.dbfile)
   print "Starting server...\n"

   GeoHandler = GeoEarthHandler if options.earth else GeoMapHandler
   httpd = BaseHTTPServer.HTTPServer((options.host, int(options.port)), GeoHandler)

   print time.asctime(), "Server Starts - %s:%s" % (options.host, options.port)

   while True:
      try:
         httpd.handle_request()
      except KeyboardInterrupt:
         break

   print
   httpd.server_close()
   print time.asctime(), "GeoMap Server Stops - %s:%s" % (options.host, options.port)











