import pygeoclients
from pygeoclients import db
from sqlalchemy import MetaData, Table
from optparse import OptionParser
from optparse import SUPPRESS_USAGE
import sys

if __name__ == "__main__":
   parser = OptionParser(usage=SUPPRESS_USAGE)
   print "Polyglot v.%s\nBy %s\n" % (pygeoclients.__version__, pygeoclients.__author__)
   parser.add_option("-s", "--source", dest="source_db",
                     help="The source database connection (ex. 'sqlite:///db.db').",
                     metavar="SOURCEDB")

   parser.add_option("-d", "--dest", dest="dest_db",
                     help="The destination database (default is 'sqlite:///db.db').",
                     metavar="DESTDB", default="sqlite:///db.db")
   
   parser.add_option("-f", "--from", dest="fro",
                     help="The table and field source (ex. clients.address).",
                     metavar="FROM")
   
   parser.add_option("-a", "--append", dest="append",
                     help="Append data to the destination database (default).",
                     default=True, action="store_true")

   parser.add_option("-r", "--reset", dest="reset",
                     help="Clear the destination database before import data.",
                     default=False, action="store_true")

   (options, args) = parser.parse_args()

   if options.append and options.reset:
      parser.error("The append and add data modes are mutually exclusive !")
      parser.print_help()
      sys.exit()

   if not options.fro:
      parser.error("You must specify the table and field source !")
      parser.print_help()
      sys.exit()

   if not options.source_db:
      parser.error("You must specify the source database !")
      parser.print_help()
      sys.exit()

   sengine, sfactory = db.create_factories(options.source_db)
   source_session = sfactory()
   dengine, dfactory = db.create_factories(options.dest_db)
   dest_session = dfactory()
   stablename = options.fro.split(".")

   smeta = MetaData(bind=sengine)
   stable = Table(stablename[0], smeta, autoload=True)

   if options.reset:
      db.reset_database(dengine)

   for record in source_session.query(stable):
      address = getattr(record, stablename[1])
      new_rec = db.AddressGeolocation(address)
      dest_session.add(new_rec)
      print "Address [%s] imported !" % address
   
   source_session.close()
   dest_session.commit()
   dest_session.close()














